'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _testSetup = require('./test-setup');

var _libcontribute = require('../src/libcontribute');

describe('LibraryContributor', function () {
	var libraryDirectory = 'a/b/c';
	var libraryName = 'abcd';
	var repo = void 0,
	    client = void 0,
	    sut = void 0;

	beforeEach(function () {
		repo = {};
		client = {};
		sut = new _libcontribute.LibraryContributor({ repo: repo, client: client });
	});

	it('saves the client and repo', function () {
		(0, _testSetup.expect)(sut.repo).to.be.deep.equal(repo);
		(0, _testSetup.expect)(sut.client).to.be.deep.equal(client);
	});

	describe('_contribute()', function () {
		it('uses the client.contribute() method to contribute a stream', function () {
			client.contributeLibrary = _testSetup.sinon.stub();
			var stream = 'stream';
			sut._contribute(libraryName, stream);
			(0, _testSetup.expect)(client.contributeLibrary).to.have.been.calledWith(stream);
			(0, _testSetup.expect)(client.contributeLibrary).to.have.been.calledOnce;
		});
	});

	describe('_buildContributePromise', function () {
		it('tars the directory but doesn\'t contribute for a dry run', function () {
			sut.targzdir = _testSetup.sinon.stub().resolves({});
			sut._contribute = _testSetup.sinon.stub();
			var dryRun = true;
			var exercise = sut._buildContributePromise(libraryDirectory, libraryName, dryRun);
			var validate = function validate() {
				(0, _testSetup.expect)(sut.targzdir).to.be.calledWith(libraryDirectory);
				(0, _testSetup.expect)(sut._contribute).to.not.have.been.called;
			};
			return exercise.then(validate);
		});

		it('tars the directory and contributees', function () {
			var pipe = 'pipe';
			sut.targzdir = _testSetup.sinon.stub().resolves(pipe);
			sut._contribute = _testSetup.sinon.stub();
			var dryRun = false;
			var exercise = sut._buildContributePromise(libraryDirectory, libraryName, dryRun);
			var validate = function validate() {
				(0, _testSetup.expect)(sut.targzdir).to.be.calledWith(libraryDirectory);
				(0, _testSetup.expect)(sut._contribute).to.have.been.calledWith(libraryName, pipe);
			};
			return exercise.then(validate);
		});
	});

	describe('_buildNotifyPromise', function () {
		function expectPromise(callbackPromise, callbackResult) {
			var notify = 'notifyMessage';
			var result = 'result';
			var promise = _promise2.default.resolve(result);
			// duplicate the promise since sinon seems to detect that the original promise has
			// been resolved and uses the resolved value rather than the original promise for
			// call matching.
			var promise2 = _promise2.default.resolve(result);
			var other1 = 'one';
			var other2 = 'two';
			var callback = _testSetup.sinon.stub().returns(callbackPromise);

			return sut._buildNotifyPromise(callback, notify, promise, other1, other2).then(function (actualResult) {
				(0, _testSetup.expect)(callback).to.have.been.calledWithMatch(notify, promise2, other1, other2);
				(0, _testSetup.expect)(actualResult).to.be.equal(callbackPromise ? callbackResult : result);
			});
		}

		it('returns the original promise when the callback returns a falsey value', function () {
			return expectPromise(false);
		});

		it('returns the overridden promise when the callback returns a truthy value', function () {
			return expectPromise(_promise2.default.resolve(123), 123);
		});
	});

	describe('_buildValidatePromise', function () {
		describe('builds a promise that calls _validateLibrary', function () {
			afterEach(function () {
				(0, _testSetup.expect)(sut._validateLibrary).to.have.been.calledWith(repo, libraryName);
			});
		});

		function expectValidateResultSuccess(validateResult) {
			sut._validateLibrary = _testSetup.sinon.stub().resolves(validateResult);
			var exercise = function exercise() {
				return sut._buildValidatePromise(libraryName);
			};
			var validate = function validate() {};
			return exercise().then(validate);
		}

		it('validates with an empty result', function () {
			return expectValidateResultSuccess(undefined);
		});
		it('validates with a valid result', function () {
			return expectValidateResultSuccess({ valid: true });
		});
		it('fails with a invalid result and no details', function () {
			var validationResult = { valid: false };
			var verifyError = function verifyError() {
				throw Error('expected exception');
			};
			var verify = function verify(error) {
				(0, _testSetup.expect)(error.validate).to.be.deep.equal(validationResult);
				(0, _testSetup.expect)(error.message).to.be.equal('Library is not valid. ');
			};
			return expectValidateResultSuccess(validationResult).then(verifyError).catch(verify);
		});

		it('fails with a invalid result and validation details', function () {
			var validationResult = { valid: false, errors: { sympathy: 'is needed' } };
			var verifyError = function verifyError() {
				throw Error('expected exception');
			};
			var verify = function verify(error) {
				(0, _testSetup.expect)(error.validate).to.be.deep.equal(validationResult);
				(0, _testSetup.expect)(error.message).to.be.equal('Library is not valid. sympathy is needed');
			};
			return expectValidateResultSuccess(validationResult).then(verifyError).catch(verify);
		});
	});

	describe('_doContributeLibrary', function () {
		it('calls _buildContributePromise', function () {
			var callback = _testSetup.sinon.stub();
			sut._buildContributePromise = _testSetup.sinon.stub().resolves();
			sut._buildNotifyPromise = _testSetup.sinon.stub().resolves();

			var library = { name: libraryName };
			var dryRun = 'dryRun';

			var exercise = sut._doContributeLibrary(callback, library, libraryDirectory, dryRun);
			var validate = function validate() {
				(0, _testSetup.expect)(sut._buildContributePromise).to.be.calledWith(libraryDirectory, libraryName, dryRun);
				(0, _testSetup.expect)(callback).to.be.calledWith('contributeComplete', library);
			};
			return exercise.then(validate);
		});
	});

	describe('contribute', function () {
		var contributeResult = 'contributeResult';
		beforeEach(function () {
			repo.libraryDirectory = _testSetup.sinon.stub().returns(libraryDirectory);
			sut._doContribute = _testSetup.sinon.stub().returns(contributeResult);
		});

		it('calls repo.libraryDirectory and passes that to _doContribute', function () {
			var callback = _testSetup.sinon.stub();
			var dryRun = 'dryRun';
			var result = sut.contribute(callback, libraryName, dryRun);
			(0, _testSetup.expect)(result).to.be.equal(contributeResult);
			(0, _testSetup.expect)(sut._doContribute).to.have.been.calledWith(callback, libraryName, libraryDirectory, dryRun);
		});

		afterEach(function () {
			(0, _testSetup.expect)(repo.libraryDirectory).to.have.been.calledWith(libraryName);
		});
	});

	describe('_doContribute', function () {
		var callback = _testSetup.sinon.stub();

		it('verify happy path', function () {
			var expectedResult = 123;
			var validatePromise = _promise2.default.resolve({ valid: true });
			var setup = function setup() {
				sut._buildValidatePromise = _testSetup.sinon.stub().returns(validatePromise);
				sut._buildNotifyPromise = _testSetup.sinon.stub().resolves(validatePromise);
				sut._doContributeDirect = _testSetup.sinon.stub().resolves(expectedResult);
			};
			var dryRun = false;
			var exercise = function exercise() {
				return sut._doContribute(callback, libraryName, libraryDirectory, dryRun);
			};
			var verify = function verify(result) {
				(0, _testSetup.expect)(sut._buildValidatePromise).to.have.been.calledWith(libraryName);
				(0, _testSetup.expect)(sut._buildNotifyPromise).to.have.been.calledWith(callback, 'validatingLibrary', validatePromise, libraryDirectory);
				(0, _testSetup.expect)(sut._doContributeDirect).to.have.been.calledWith(callback, libraryName, libraryDirectory, dryRun);
				(0, _testSetup.expect)(result).equals(expectedResult);
			};

			return _promise2.default.resolve().then(setup).then(exercise).then(verify);
		});

		it('propagates validation failure', function () {
			var validationError = new Error('didn\'t validate');
			var validatePromise = _promise2.default.reject(validationError);
			var setup = function setup() {
				sut._buildValidatePromise = _testSetup.sinon.stub().returns(validatePromise);
				sut._buildNotifyPromise = _testSetup.sinon.stub().resolves(validatePromise);
				sut._doContributeDirect = _testSetup.sinon.stub();
			};

			var dryRun = false;
			var exercise = function exercise() {
				return sut._doContribute(callback, libraryName, libraryDirectory, dryRun);
			};
			var verify = function verify(result) {
				(0, _testSetup.expect)(sut._doContributeDirect).to.not.have.been.called;
				(0, _testSetup.expect)(result).to.be.equal(validationError);
			};
			var verifyFail = function verifyFail() {
				throw Error('expected exception');
			};

			return _promise2.default.resolve().then(setup).then(exercise).then(verifyFail).catch(verify);
		});
	});

	describe('targzdir', function () {
		var tmpdirobj = void 0;
		var path = require('path');
		var fs = require('fs');

		beforeEach(function () {
			var tmp = require('tmp');
			tmp.setGracefulCleanup();
			tmpdirobj = tmp.dirSync();
		});

		afterEach(function () {
			//require('tmp').removeCallback();
		});

		var files = {
			'src/file.cpp': 'void f() { return 42;}',
			'.gitignore': 'me',
			'.git/blah': 'stuff'
		};

		function createFile(tmpdir, name, content) {
			var dir = path.join(tmpdir, path.dirname(name));
			if (!fs.existsSync(dir)) {
				fs.mkdirSync(dir);
			}
			fs.writeFileSync(path.join(tmpdir, name), content);
		}

		function verifyFile(tmpdir, name, content) {
			var file = path.join(tmpdir, name);
			if (name.startsWith('.git/')) {
				(0, _testSetup.expect)(fs.existsSync(file), 'file ' + file + ' should not exist').to.eql(false);
			} else {
				(0, _testSetup.expect)(fs.existsSync(file), 'file ' + file + ' should exist').to.eql(true);
				(0, _testSetup.expect)(fs.readFileSync(file, 'utf-8'), 'expected file ' + file + ' content').to.eql(content);
			}
		}

		it('makes a targz file that does not include the .git directory', function () {
			// given
			var tmpdir = tmpdirobj.name;
			(0, _keys2.default)(files).forEach(function (key) {
				return createFile(tmpdir, key, files[key]);
			});
			var sut = new _libcontribute.LibraryContributor({}, {});
			return sut.targzdir(tmpdir).then(function (stream) {
				var resultdir = path.join(tmpdir, 'result');
				fs.mkdirSync(resultdir);

				var tarfs = require('tar-fs');
				var gunzip = require('gunzip-maybe');
				return new _promise2.default(function (fulfill, reject) {
					var extract = tarfs.extract(resultdir);
					extract.on('finish', fulfill);
					extract.on('error', reject);
					stream.pipe(gunzip()).pipe(extract);
				}).then(function () {
					(0, _keys2.default)(files).forEach(function (key) {
						return verifyFile(resultdir, key, files[key]);
					});
				});
			});
		});
	});
});
//# sourceMappingURL=libcontribute.spec.js.map