'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _librepo = require('../src/librepo');

var _verror = require('verror');

var _verror2 = _interopRequireDefault(_verror);

var _testSetup = require('./test-setup');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Writable = require('stream').Writable; /*
                                            ******************************************************************************
                                            Copyright (c) 2016 Particle Industries, Inc.  All rights reserved.
                                           
                                            This program is free software; you can redistribute it and/or
                                            modify it under the terms of the GNU Lesser General Public
                                            License as published by the Free Software Foundation, either
                                            version 3 of the License, or (at your option) any later version.
                                           
                                            This program is distributed in the hope that it will be useful,
                                            but WITHOUT ANY WARRANTY; without even the implied warranty of
                                            MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
                                            Lesser General Public License for more details.
                                           
                                            You should have received a copy of the GNU Lesser General Public
                                            License along with this program; if not, see <http://www.gnu.org/licenses/>.
                                            ******************************************************************************
                                            */

describe('LibraryManager', function () {

	describe('Library Errors', function () {

		describe('LibraryRepositoryError', function () {
			it('has repo property', function () {
				var repo = {};
				var sut = new _librepo.LibraryRepositoryError(repo);
				(0, _testSetup.expect)(sut.repo).to.equal(repo);
			});

			it('can use instanceof', function () {
				var sut = new _librepo.LibraryRepositoryError();
				// using instanceof doesn't work
				// expect(sut).to.be.an.instanceof(LibraryRepositoryError);
				// expect(sut instanceof LibraryRepositoryError).to.be.true;
				(0, _testSetup.expect)(sut.name).to.be.string('LibraryRepositoryError');
			});
		});

		describe('LibraryNotFoundError', function () {
			it('has repo and library properties', function () {
				var repo = {};
				var library = 'uberlib';
				var sut = new _librepo.LibraryNotFoundError(repo, library);
				(0, _testSetup.expect)(sut.library).to.equal(library);
				(0, _testSetup.expect)(sut.repo).to.equal(repo);
			});

			it('has a message', function () {
				var sut = new _librepo.LibraryNotFoundError('uberrepo', 'uberlib');
				(0, _testSetup.expect)(sut.toString()).to.equal('LibraryNotFoundError: library \'uberlib\' not found in repo \'uberrepo\'.');
			});
		});

		describe('LibraryFormatError', function () {
			it('works', function () {
				var repo = {};
				var library = 'uberlib';
				var sut = new _librepo.LibraryFormatError(repo, library, new _verror2.default('bad mojo'));

				(0, _testSetup.expect)(sut.library).to.equal(library);
				(0, _testSetup.expect)(sut.repo).to.equal(repo);
				(0, _testSetup.expect)(sut.message).to.equal(': bad mojo');
				(0, _testSetup.expect)(sut.name).to.equal('LibraryFormatError');
			});
		});
	});

	describe('Library', function () {

		it('has a name', function () {
			var sut = new _librepo.Library('Borgian');
			(0, _testSetup.expect)(sut.name).to.equal('Borgian');
		});

		it('has no files', function () {
			var sut = new _librepo.Library();
			return (0, _testSetup.expect)(sut.files()).to.eventually.have.length(0);
		});

		it('has no definition', function () {
			var sut = new _librepo.Library();
			return (0, _testSetup.expect)(sut.definition()).to.eventually.be.rejected;
		});
	});

	describe('LibraryFile', function () {
		it('constructs', function () {
			var name = {};
			var type = {};
			var ext = {};
			var sut = new _librepo.LibraryFile(name, type, ext);
			(0, _testSetup.expect)(sut.name).to.equal(name);
			(0, _testSetup.expect)(sut.kind).to.equal(type);
			(0, _testSetup.expect)(sut.extension).to.equal(ext);
		});

		it('has streamable content', function () {
			var sut = new _librepo.LibraryFile('name', 'type', 'ext');
			var stream = { end: _testSetup.sinon.spy() };
			var p = sut.content(stream);
			(0, _testSetup.expect)(stream.end).to.be.calledOnce;
			return (0, _testSetup.expect)(p).to.eventually.equal(stream);
		});
	});

	describe('LibraryRepository', function () {
		it('raises an error when fetching a library by name', function () {
			var sut = new _librepo.LibraryRepository();
			return (0, _testSetup.expect)(sut.fetch('uberlib')).eventually.rejected.deep.equal(new _librepo.LibraryNotFoundError(sut, 'uberlib'));
		});

		it('has no libraries', function () {
			var sut = new _librepo.LibraryRepository();
			return (0, _testSetup.expect)(sut.names()).eventually.to.have.length(0);
		});
	});

	describe('MemoryLibraryFile', function () {
		it('constructs', function () {
			var sut = new _librepo.MemoryLibraryFile('file', 'nice', 'ext', 'lots of content here', '123');
			(0, _testSetup.expect)(sut.name).to.be.string('file');
			(0, _testSetup.expect)(sut.kind).to.be.string('nice');
			(0, _testSetup.expect)(sut.extension).to.be.string('ext');
			(0, _testSetup.expect)(sut.string_content).to.be.string('lots of content here');
			(0, _testSetup.expect)(sut.id).to.be.string('123');
		});

		it('straems content', function () {
			var sut = new _librepo.MemoryLibraryFile('file', 'nice', 'ext', 'lots of content here', '123');
			var result = '';
			var ws = Writable(); // eslint-disable-line new-cap
			ws._write = function (chunk, enc, next) {
				result += chunk;
				next();
			};
			ws.on('end', function () {
				(0, _testSetup.expect)(result).to.be.equal('lots of content here');
			});
			sut.content(ws);
		});
	});

	describe('AbstractLibrary', function () {
		it('retrieves the initial definition from the repo and caches it thereafter', function () {
			var repo = {};
			repo.definition = _testSetup.sinon.stub();
			var defn = 'definition';
			repo.definition.returns(_promise2.default.resolve(defn));

			var sut = new _librepo.AbstractLibrary('name', {}, repo);
			_testSetup.sinon.spy(sut, 'processDefinition');
			var result = sut.definition();

			var promise = result.then(function (def) {
				(0, _testSetup.expect)(def).to.be.equal(defn);
				(0, _testSetup.expect)(repo.definition).to.be.calledOnce.calledWith(sut);
				(0, _testSetup.expect)(sut.processDefinition).to.be.calledOnce.calledWith(defn);
				return def;
			}).then(function () {
				return sut.definition();
			}).then(function (def) {
				(0, _testSetup.expect)(def).to.be.equal(defn);
				(0, _testSetup.expect)(repo.definition).to.be.calledOnce;
				return def;
			});
			return (0, _testSetup.expect)(promise).to.be.eventually.equal(defn);
		});

		it('retrieves the initial files from the repo and caches them thereafter', function () {
			var repo = {};
			repo.files = _testSetup.sinon.stub();
			var files = 'files';
			repo.files.returns(_promise2.default.resolve(files));

			var sut = new _librepo.AbstractLibrary('name', {}, repo);
			_testSetup.sinon.spy(sut, 'processFiles');
			var result = sut.files();

			var promise = result.then(function (f) {
				(0, _testSetup.expect)(f).to.be.equal(files);
				(0, _testSetup.expect)(repo.files).to.be.calledOnce.and.calledWith(sut);
				(0, _testSetup.expect)(sut.processFiles).to.be.calledOnce.and.calledWith(files);
				return f;
			}).then(function () {
				return sut.files();
			}).then(function (f) {
				(0, _testSetup.expect)(f).to.be.equal(files);
				(0, _testSetup.expect)(repo.files).to.be.calledOnce;
				return f;
			});

			return (0, _testSetup.expect)(promise).to.be.eventually.equal(files);
		});

		it('propagates errors when retrieving the definition', function () {
			var repo = {};
			repo.definition = _testSetup.sinon.stub();
			repo.definition.returns(_promise2.default.resolve().then(function () {
				throw 'keep calm and carry on';
			}));
			var sut = new _librepo.AbstractLibrary('name', {}, repo);
			return (0, _testSetup.expect)(sut.definition()).to.be.rejectedWith('keep calm and carry on');
		});

		it('propagates errors when retrieving the files', function () {
			var repo = {};
			repo.files = _testSetup.sinon.stub();
			repo.files.returns(_promise2.default.resolve().then(function () {
				throw 'aliens are coming';
			}));
			var sut = new _librepo.AbstractLibrary('name', {}, repo);
			return (0, _testSetup.expect)(sut.files()).to.be.rejectedWith('aliens are coming');
		});
	});

	describe('AbstractLibraryRepository', function () {
		it('returns an empty list of files', function () {
			var sut = new _librepo.AbstractLibraryRepository();
			(0, _testSetup.expect)(sut.files()).to.eventually.equal([]);
		});

		it('returns a defintiion comprising just the library name', function () {
			var sut = new _librepo.AbstractLibraryRepository();
			(0, _testSetup.expect)(sut.definition({ name: 'wombat' })).to.eventually.equal('wombat');
		});
	});
});
//# sourceMappingURL=librepo.spec.js.map