'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _testSetup = require('./test-setup');

var _librepo_build = require('../src/librepo_build');

var _Agent = require('particle-api-js/lib/Agent');

var _Agent2 = _interopRequireDefault(_Agent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Build', function () {

	describe('Library Repo', function () {
		it('is constructed with the endpoint url', function () {
			var endpoint = 'abcde';
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: endpoint });
			(0, _testSetup.expect)(sut).has.property('endpoint').which.is.equal(endpoint);
			(0, _testSetup.expect)(sut).has.property('agent').which.is.instanceOf(_Agent2.default);
		});

		it('fetches the library index using an agent', _testSetup.sinon.test(function () {
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: 'abc.com/' });
			var agent = _testSetup.sinon.stub(sut.agent);
			agent.get.returns(_promise2.default.reject('unknown args'));
			agent.get.withArgs('abc.com/libs.json').returns(_promise2.default.resolve({ body: '123' }));
			return (0, _testSetup.expect)(sut.index()).to.eventually.equal('123');
		}));

		it('rubs the lotion on its skin or it gets the hose again', function () {
			var sut = _promise2.default.reject('the lotion');
			(0, _testSetup.expect)(sut).eventually.reject;
		});

		it('fetches library names', _testSetup.sinon.test(function () {
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: 'abc.com/' });
			var index = _testSetup.sinon.stub(sut, 'index');
			index.returns(_promise2.default.resolve([{ title: 'lib1' }, { title: 'lib2' }]));
			return (0, _testSetup.expect)(sut.names()).eventually.deep.equal(['lib1', 'lib2']);
		}));

		it('fetches known libraries', _testSetup.sinon.test(function () {
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: '$$$/' });
			var get = _testSetup.sinon.stub(sut, 'get');
			var lib = { title: 'uberlib', id: '123' };
			get.returns(_promise2.default.reject('unknown args'));
			get.withArgs('libs.json', { name: 'uberlib' }).returns(_promise2.default.resolve([lib]));
			return (0, _testSetup.expect)(sut.fetch('uberlib')).eventually.deep.equal(new _librepo_build.BuildLibrary('uberlib', lib, '123', sut));
		}));

		it('throws exception for unknown libraries', _testSetup.sinon.test(function () {
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: '$$$/' });
			var get = _testSetup.sinon.stub(sut, 'get');
			get.returns(_promise2.default.reject('unknown args'));
			get.withArgs('libs.json', { name: 'uberlib' }).returns(_promise2.default.resolve([]));
			return (0, _testSetup.expect)(sut.fetch('uberlib')).eventually.rejectedWith('library \'uberlib\' not found in repo \'[object Object]\'.');
		}));

		it('uses libraryId to fetch the library id', function () {
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: '$$$/' });
			(0, _testSetup.expect)(sut.libraryId({ id: '123' })).to.equal('123');
		});

		it('retrieves files for a library via the id and a get request', function () {
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: '/root' });
			sut.get = _testSetup.sinon.stub();
			sut.get.returns('abc');
			(0, _testSetup.expect)(sut.files({ id: 123 })).to.equal('abc');
			(0, _testSetup.expect)(sut.get).calledOnce.calledWith('libs/123/tabs.json');
		});

		it('retrieves the definition for a library via the id and a get request', function () {
			var sut = new _librepo_build.BuildLibraryRepository({ endpoint: '/root' });
			sut.get = _testSetup.sinon.stub();
			sut.get.returns('abc');
			(0, _testSetup.expect)(sut.definition({ id: 123 })).to.equal('abc');
			(0, _testSetup.expect)(sut.get).calledOnce.calledWith('libs/123/definition.json');
		});
	});

	describe('Library', function () {
		it('throws an error if no id is provided', function () {
			(0, _testSetup.expect)(function () {
				return new _librepo_build.BuildLibrary('name', {}, '', 'repo');
			}).to.throw('LibraryFormatError: no id');
		});

		it('constructs with an id', function () {
			var sut = new _librepo_build.BuildLibrary('name', { id: '123' }, 'repo');
			(0, _testSetup.expect)(sut.metadata.id).to.be.equal('123');
		});

		it('implements processFiles via tabsToFiles', function () {
			var sut = new _librepo_build.BuildLibrary('name', { id: '123' }, 'repo');
			sut.tabsToFiles = _testSetup.sinon.stub();
			sut.tabsToFiles.returns('result');
			(0, _testSetup.expect)(sut.processFiles(['abcd'])).to.equal('result');
			(0, _testSetup.expect)(sut.tabsToFiles).calledOnce.calledWith(['abcd']);
		});

		it('can convert tabs to files', function () {
			var tabs = [{ title: 'title1', kind: 'kind1', extension: 'ext1', content: 'cnt1', id: 'id1' }, { title: 'title2', kind: 'kind2', extension: 'ext2', content: 'cnt2', id: 'id2' }];

			var sut = new _librepo_build.BuildLibrary('name', { id: '123' }, 'repo');
			var files = sut.tabsToFiles(tabs);
			(0, _testSetup.expect)(files).to.have.length(tabs.length);
			files.forEach(function (file, index) {
				var tab = tabs[index];
				(0, _testSetup.expect)(file.string_content).to.equal(tab.content);
				(0, _testSetup.expect)(file.id).to.equal(tab.id);
				(0, _testSetup.expect)(file.name).to.equal(tab.title);
				(0, _testSetup.expect)(file.kind).to.equal(tab.kind);
				(0, _testSetup.expect)(file.extension).to.equal(tab.extension);
			});
		});
	});
}); /*
     ******************************************************************************
     Copyright (c) 2016 Particle Industries, Inc.  All rights reserved.
    
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation, either
     version 3 of the License, or (at your option) any later version.
    
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
    
     You should have received a copy of the GNU Lesser General Public
     License along with this program; if not, see <http://www.gnu.org/licenses/>.
     ******************************************************************************
     */
//# sourceMappingURL=librepo_build.spec.js.map