'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _testSetup = require('./test-setup');

var _librepo_cloud = require('../src/librepo_cloud');

/*
 ******************************************************************************
 Copyright (c) 2016 Particle Industries, Inc.  All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation, either
 version 3 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this program; if not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************
 */

var fs = require('fs');
var path = require('path');
var mockfs = require('mock-fs');
require('es6-promise').polyfill();
require('promise.prototype.finally');

describe('CloudLibraryRepository', function () {

	var client = { api: {} };
	var sut = new _librepo_cloud.CloudLibraryRepository({ auth: 'auth', client: client });

	it('can be instantiated with auth token', function () {
		var sut = new _librepo_cloud.CloudLibraryRepository({ auth: 'auth' });
		(0, _testSetup.expect)(sut).to.be.ok;
		(0, _testSetup.expect)(sut).to.have.property('client').that.is.ok;
		(0, _testSetup.expect)(sut).to.have.property('api').that.is.equal(sut.client.api);
		(0, _testSetup.expect)(sut).to.have.property('auth').that.is.equal('auth');
	});

	it('can be instantiated without auth token', function () {
		var sut = new _librepo_cloud.CloudLibraryRepository({});
		(0, _testSetup.expect)(sut).to.be.ok;
		(0, _testSetup.expect)(sut).to.have.property('client').that.is.ok;
		(0, _testSetup.expect)(sut).to.have.property('api').that.is.equal(sut.client.api);
		(0, _testSetup.expect)(sut).to.have.property('auth').that.is.undefined;
	});

	it('can be instantiated with a client', function () {
		var sut = new _librepo_cloud.CloudLibraryRepository({ auth: 'auth', client: client });
		(0, _testSetup.expect)(sut).to.have.property('client').that.is.equal(client);
		(0, _testSetup.expect)(sut).to.have.property('api').that.is.equal(client.api);
		(0, _testSetup.expect)(sut).to.have.property('auth').that.is.equal('auth');
	});

	it('delegates getLibrary to the client', function () {
		client.library = _testSetup.sinon.stub();
		sut._getLibrary('somelib', '1.2.3');
		(0, _testSetup.expect)(client.library).to.be.calledWith('somelib', { version: '1.2.3' });
	});

	it('can create a new library', function () {
		var lib = sut._createLibrary('name', 'meta');
		(0, _testSetup.expect)(lib).to.be.deep.equal(new _librepo_cloud.CloudLibrary('name', 'meta', sut));
	});

	it('delegates fetch to _getLibrary and _createLibrary', function () {
		var lib = { name: 'mylib' };
		sut._getLibrary = _testSetup.sinon.stub().returns(_promise2.default.resolve(lib));
		return sut.fetch('mylib').then(function (result) {
			(0, _testSetup.expect)(result).to.deep.equal(new _librepo_cloud.CloudLibrary('mylib', lib, sut));
			(0, _testSetup.expect)(sut._getLibrary).to.have.been.calledWith('mylib');
		});
	});

	it('delegates index() to client.libraries()', function () {
		var libs = [{ name: 'lib1' }, { name: 'lib2' }];
		client.libraries = _testSetup.sinon.stub().returns(_promise2.default.resolve(libs));
		return sut.index().then(function (result) {
			(0, _testSetup.expect)(result).to.be.deep.equal(libs);
			(0, _testSetup.expect)(client.libraries).to.have.been.calledWith();
		});
	});

	it('delegates names() to index and extractNames', function () {
		var libs = [{ name: 'lib1' }, { name: 'lib2' }];
		sut.index = _testSetup.sinon.stub().returns(_promise2.default.resolve(libs));
		return sut.names().then(function (names) {
			(0, _testSetup.expect)(sut.index).to.be.calledWith();
			(0, _testSetup.expect)(names).to.be.deep.equal(['lib1', 'lib2']);
		});
	});

	it('definition fetches the library metadata', function () {
		var lib = { metadata: 'abcd' };
		(0, _testSetup.expect)(sut.definition(lib)).to.eventually.be.equal('abcd');
	});

	it('can expand a tar.gz file to a directory', function () {
		var buffer = fs.readFileSync(path.join(__dirname, 'fixtures', 'tarball.tar.gz'));
		var lib = {};
		lib.download = _testSetup.sinon.stub().returns(_promise2.default.resolve(buffer));
		var sut = new _librepo_cloud.CloudLibrary('abcd', lib);

		mockfs({ '/': {} });
		// this isn't a pure unit test, but is simpler to code than mocking the tar.gz functionality.
		return sut.copyTo('/newlib').then(function (lib) {
			(0, _testSetup.expect)(lib).to.be.deep.equal(sut);
			(0, _testSetup.expect)('/newlib/library.properties').to.be.a.file;
			(0, _testSetup.expect)('/newlib/project.properties').to.not.be.a.file;
			(0, _testSetup.expect)('/newlib/project.properties').to.not.be.a.file;
			(0, _testSetup.expect)('/newlib/src/neopixel.cpp').to.be.a.file;
			(0, _testSetup.expect)('/newlib/src/neopixel.h').to.be.a.file;
		}).finally(function () {
			mockfs.restore();
		});
	});

	it('ignores symlinks ', function () {
		// todo - build a tar.gz containing symblinks

	});

	describe('mockfs', function () {
		it('fails when the tar.gz is not valid', function () {
			var buffer = new Buffer(2000);
			buffer.fill(0);
			var lib = { download: _testSetup.sinon.stub().returns(_promise2.default.resolve(buffer)) };
			var sut = new _librepo_cloud.CloudLibrary('abcd', lib);
			(0, _testSetup.expect)(sut.copyTo('tmp')).to.eventually.reject;
		});
	});
});
//# sourceMappingURL=librepo_cloud.spec.js.map