'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _arguments = arguments; /*
                             ******************************************************************************
                             Copyright (c) 2016 Particle Industries, Inc.  All rights reserved.
                            
                             This program is free software; you can redistribute it and/or
                             modify it under the terms of the GNU Lesser General Public
                             License as published by the Free Software Foundation, either
                             version 3 of the License, or (at your option) any later version.
                            
                             This program is distributed in the hope that it will be useful,
                             but WITHOUT ANY WARRANTY; without even the implied warranty of
                             MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
                             Lesser General Public License for more details.
                            
                             You should have received a copy of the GNU Lesser General Public
                             License along with this program; if not, see <http://www.gnu.org/licenses/>.
                             ******************************************************************************
                             */

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _testSetup = require('./test-setup');

var _librepo_fs = require('../src/librepo_fs');

var _libcontribute = require('../src/libcontribute');

var _librepo_fs_mock = require('./librepo_fs_mock.spec');

var fs = require('fs');
var path = require('path');

describe('File System', function () {
	// this is an integration test, but since everything needed is available locally
	// and it's pretty quick we run it here in the unit test suite

	var tmp = require('tmp');
	var fse = require('fs-extra');
	var dircomp = require('dir-compare');
	tmp.setGracefulCleanup();

	var testdata = path.join(__dirname, '..', 'resources', 'libraries');

	it('can detect a v1 library', function () {
		var sut = new _librepo_fs.FileSystemLibraryRepository(testdata);
		return (0, _testSetup.expect)(sut.getLibraryLayout('library-v1')).to.eventually.be.equal(1);
	});

	it('can detect a v2 library', function () {
		var sut = new _librepo_fs.FileSystemLibraryRepository(testdata);
		return (0, _testSetup.expect)(sut.getLibraryLayout('library-v2')).to.eventually.be.equal(2);
	});

	it('rasies an exception attempting to migrate a v2 library to v1', function () {
		var sut = new _librepo_fs.FileSystemLibraryRepository(testdata);
		(0, _testSetup.expect)(sut.setLibraryLayout('library-v2', 1)).eventually.rejected;
	});

	it('is a no-op to migrate a v2 library to v2', function () {
		var sut = new _librepo_fs.FileSystemLibraryRepository(path.join(testdata, 'library-v2'));
		(0, _testSetup.expect)(sut.setLibraryLayout('library-v2', 2)).eventually.not.rejected;
	});

	describe('direct naming strategy', function () {
		var sut = new _librepo_fs.FileSystemLibraryRepository(testdata + '/library-v2', _librepo_fs.FileSystemNamingStrategy.DIRECT);

		it('is a no-op to migrate a v2 library to v2 with direct naming strategy, default name', function () {
			return (0, _testSetup.expect)(sut.setLibraryLayout('', 2)).eventually.not.rejected;
		});

		it('is a no-op to migrate a v2 library to v2 with direct naming strategy, explicit name', function () {
			return (0, _testSetup.expect)(sut.setLibraryLayout('uber-library-example', 2)).eventually.not.rejected;
		});

		it('lists the library name as the only library', function () {
			return (0, _testSetup.expect)(sut.names()).to.eventually.deep.equal(['uber-library-example']);
		});

		it('fetches the library via its name', function () {
			return (0, _testSetup.expect)(sut.fetch('uber-library-example')).to.eventually.have.property('name').equal('uber-library-example');
		});

		it('fetches the library via the empty name', function () {
			return (0, _testSetup.expect)(sut.fetch('')).to.eventually.have.property('name').equal('uber-library-example');
		});
	});

	function assertMigrate(v1, v2, naming) {
		var v1data = path.join(testdata, v1);
		var v2data = path.join(testdata, v2);

		var tmpobj = tmp.dirSync();
		var dir = tmpobj.name;
		var name = 'testlib';
		var libdir = path.join(dir, name);

		fs.mkdirSync(libdir);

		fse.copySync(v1data, libdir);
		var comp1 = dircomp.compareSync(libdir, v1data, { compareContent: true });
		(0, _testSetup.expect)(comp1.same).to.be.true;

		var sut = new _librepo_fs.FileSystemLibraryRepository(dir, naming);
		return sut.setLibraryLayout(name, 2).then(function () {
			var comp2 = dircomp.compareSync(libdir, v2data, { compareContent: true });
			if (!comp2.same) {
				//const unequal = comp2.diffSet.filter(item => item.state!=='equal');
			}
			(0, _testSetup.expect)(comp2.same).to.be.true;
		});
	}

	it('can migrate a full v1 library to v2 format', function () {
		return assertMigrate('library-v1', 'library-v2');
	});

	it('can migrate a v1 library without tests to v2 format', function () {
		return assertMigrate('library-v1-notests', 'library-v2-notests');
	});

	it('can migrate a v1 library without examples to v2 format', function () {
		return assertMigrate('library-v1-noexamples', 'library-v2-noexamples');
	});

	describe('LibraryContributor', function () {

		it('fails if the library does not validate', function () {
			var name = 'fred';
			var client = undefined;
			var tmpobj = tmp.dirSync();
			var dir = tmpobj.name;
			var repo = new _librepo_fs.FileSystemLibraryRepository(dir);
			var lib = (0, _librepo_fs_mock.makeTestLib)(name, '1.2.3');
			var sut = new _libcontribute.LibraryContributor({ repo: repo, client: client });

			sut._contribute = _testSetup.sinon.stub();

			var result = repo.add(lib, 2).then(function () {
				return sut.contribute(function () {}, name);
			});
			return (0, _testSetup.expect)(result).to.eventually.be.rejected;
		});

		it('can contribute a library as a tarball', function () {
			var name = 'fred';
			var client = undefined;

			var tmpobj = tmp.dirSync();
			var dir = tmpobj.name;
			var repo = new _librepo_fs.FileSystemLibraryRepository(dir);
			var lib = (0, _librepo_fs_mock.makeCompleteV2Lib)(name, '1.2.3');

			var sut = new _libcontribute.LibraryContributor({ repo: repo, client: client });
			var callback = _testSetup.sinon.stub();
			sut._contribute = _testSetup.sinon.stub();

			var result = repo.add(lib, 2).then(function () {
				return sut.contribute(callback, name);
			}).then(function () {
				(0, _testSetup.expect)(sut._contribute).to.have.been.calledOnce;
				(0, _testSetup.expect)(sut._contribute).to.have.been.calledWith(name);
				var pipe = sut._contribute.firstCall.args[1];

				var zlib = require('zlib');
				var tar = require('tar-stream');
				var gunzip = zlib.createGunzip();
				var extract = tar.extract();
				var names = [];
				extract.on('entry', function (header, stream, callback) {
					names.push(header.name);
					stream.on('end', function () {
						return callback();
					});
					stream.resume();
				});
				var promise = new _promise2.default(function (fulfill, reject) {
					extract.on('finish', fulfill);
					extract.on('error', reject);
					pipe.pipe(gunzip).pipe(extract);
				});
				return promise.then(function () {
					(0, _testSetup.expect)(names).to.include('README.md');
					(0, _testSetup.expect)(names).include('LICENSE.');
					(0, _testSetup.expect)(names).include('library.properties');
					(0, _testSetup.expect)(names).include('src/fred.cpp');
					(0, _testSetup.expect)(names).include('src/fred.h');

					(0, _testSetup.expect)(callback).to.have.been.calledWith('validatingLibrary');
					(0, _testSetup.expect)(callback).to.have.been.calledWith('contributingLibrary');
					(0, _testSetup.expect)(callback).to.have.been.calledWith('contributeComplete');
				});
			});
			return result;
		});

		it('can attempt to publish from a repo for real', function () {
			var repo = new _librepo_fs.FileSystemLibraryRepository('mydir');
			return (0, _testSetup.expect)(repo.contribute('abcd', {}, false, function () {})).to.eventually.be.rejected;
		});

		it('can attempt to publish from a repo as a dry run', function () {
			var repo = new _librepo_fs.FileSystemLibraryRepository('mydir');
			return (0, _testSetup.expect)(repo.contribute('abcd', {}, true, function () {
				return _arguments[1];
			})).to.eventually.be.rejected;
		});
	});

	describe('library examples', function () {
		describe('given an example via a relative path', function () {
			var example = void 0;
			var cwd = void 0;
			var files = void 0;
			var libname = 'library-v2-adapters';
			/**
    * Make the current directory the test data
    */
			beforeEach(function () {
				cwd = process.cwd();
				process.chdir(testdata);
				var examplePromise = (0, _librepo_fs.isLibraryExample)(path.join(libname, 'examples', 'blink-an-led'));

				return examplePromise.then(function (ex) {
					example = ex;
					(0, _testSetup.expect)(example).to.be.ok;
				}).then(function () {
					files = {};
					return example.buildFiles(files);
				});
			});

			afterEach(function () {
				process.chdir(cwd);
			});

			it('recognizes it as an example', function () {
				(0, _testSetup.expect)(example).to.have.property('basePath').equal(path.resolve(testdata));
				(0, _testSetup.expect)(example).to.have.property('libraryPath').equal(libname);
				(0, _testSetup.expect)(example).to.have.property('example').equal(path.join(libname, 'examples', 'blink-an-led') + path.sep);
				(0, _testSetup.expect)(files).to.have.property('map').that.is.ok;
				(0, _testSetup.expect)(files).to.have.property('basePath').that.is.equal(testdata);
			});

			function expectHasMapping(src) {
				var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : src;

				// src is relative to the library
				src = path.join(libname, src);
				(0, _testSetup.expect)(files.map).has.property(target).equal(src);
			}

			it('moves library.properties to project.properties', function () {
				expectHasMapping('library.properties', 'project.properties');
			});

			it('moves the example sources to the sources of the project', function () {
				expectHasMapping(path.join('examples', 'blink-an-led', 'blink-an-led.cpp'), path.join('src', 'blink-an-led.cpp'));
			});

			it('moves the library sources to the sources directory', function () {
				expectHasMapping(path.join('src', 'uber-library-example.cpp'));
				expectHasMapping(path.join('src', 'uber-library-example.h'));
				expectHasMapping(path.join('src', 'a-c-example.c'));
			});

			describe('LibraryExample class', function () {
				it('_isFile returns true when the filename does not end with a slash', function () {
					(0, _testSetup.expect)(example._isFile('/abcd')).to.be.true;
				});

				it('_isFile returns false when the filename does end with a slash', function () {
					(0, _testSetup.expect)(example._isFile('abcd/')).to.be.false;
				});
			});
		});
	});
});
//# sourceMappingURL=librepo_fs.spec.js.map