'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _testSetup = require('./test-setup');

var _validation = require('../src/validation');

var _librepo_fs = require('../src/librepo_fs');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('validation', function () {

	describe('validateField', function () {
		it('returns an object', function () {
			(0, _testSetup.expect)((0, _validation.validateField)('field', 'value')).to.be.ok;
		});

		function expectField(field, value, result) {
			(0, _testSetup.expect)(result.field).to.equal(field);
			(0, _testSetup.expect)(result.value).to.equal(value);
		}

		function expectValid(field, value) {
			var result = (0, _validation.validateField)(field, value);
			expectField(field, value, result);
			(0, _testSetup.expect)(result.valid).to.be.true;
		}

		function expectError(field, value, expectedMessage) {
			var result = (0, _validation.validateField)(field, value);
			expectField(field, value, result);
			(0, _testSetup.expect)(result.valid).to.be.false;
			(0, _testSetup.expect)(result.errors[field]).to.equal(expectedMessage);
		}

		it('returns valid for an unknown field', function () {
			expectValid('field', 'value');
		});

		describe('name', function () {
			var errorMessage = 'must only contain letters, numbers, dashes, underscores and plus signs.';

			it('returns valid for a valid name', function () {
				expectValid('name', 'my-lib_123');
			});

			it('returns valid for a name containing plus signs', function () {
				expectValid('name', 'my-lib_123++');
			});

			it('returns valid for a name containing plus signs at the start', function () {
				expectError('name', '+my-lib_123++', errorMessage);
			});

			it('returns error for missing name', function () {
				expectError('name', undefined, "can't be blank");
			});

			it('returns error for blank name', function () {
				expectError('name', '', "can't be blank");
			});

			it('returns error for a name starting with a dash', function () {
				expectError('name', '-help', errorMessage);
			});

			it('returns error for a name with forbidden characters', function () {
				expectError('name', 'foo@bar', errorMessage);
			});
		});

		describe('version', function () {
			it('returns valid for a valid version', function () {
				expectValid('version', '1.10.5');
			});

			it('returns error for missing version', function () {
				expectError('version', undefined, "can't be blank");
			});

			it('returns error for blank version', function () {
				expectError('version', '', "can't be blank");
			});

			it('returns error for a name with too few numbers', function () {
				expectError('version', '1.0', 'must be formatted like 1.0.0');
			});

			it('returns error for a name with too many numbers', function () {
				expectError('version', '1.0.5.1', 'must be formatted like 1.0.0');
			});

			it('returns error for a name with invalid characters', function () {
				expectError('version', 'test1', 'must be formatted like 1.0.0');
			});
		});

		describe('author', function () {
			it('returns valid for a valid author', function () {
				expectValid('author', 'Author <author@example.com>');
			});

			it('returns error for missing author', function () {
				expectError('author', undefined, "can't be blank");
			});

			it('returns error for blank name', function () {
				expectError('author', '', "can't be blank");
			});
		});
	});

	describe('validateMetadata', function () {
		it('returns an object', function () {
			(0, _testSetup.expect)((0, _validation.validateMetadata)({})).to.be.ok;
		});

		var validMetadata = {
			name: 'mylib',
			author: 'Author <author@example.com>',
			version: '1.0.0',
			sentence: 'A library'
		};

		function expectValid(metadata) {
			var result = (0, _validation.validateMetadata)(metadata);
			(0, _testSetup.expect)(result.valid).to.be.true;
		}

		function expectError(metadata) {
			var result = (0, _validation.validateMetadata)(metadata);
			(0, _testSetup.expect)(result.valid).to.be.false;

			for (var _len = arguments.length, fields = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
				fields[_key - 1] = arguments[_key];
			}

			(0, _testSetup.expect)(result.errors).to.have.all.keys(fields);
		}

		it('returns valid for valid metadata', function () {
			expectValid(validMetadata);
		});

		it('returns an error when a field is invalid', function () {
			var metadata = (0, _assign2.default)({}, validMetadata, {
				name: ''
			});

			expectError(metadata, 'name');
		});

		it('returns multiple errors when multiple fields is invalid', function () {
			var metadata = (0, _assign2.default)({}, validMetadata, {
				name: '',
				version: ''
			});

			expectError(metadata, 'name', 'version');
		});
	});

	describe('validateLibRepo', function () {
		var testdata = _path2.default.join(__dirname, '..', 'resources', 'libraries');
		function getLibrary(name) {
			return _promise2.default.resolve(new _librepo_fs.FileSystemLibraryRepository(testdata + '/' + name, _librepo_fs.FileSystemNamingStrategy.DIRECT));
		}

		it('returns valid for valid library', function () {
			var setup = function setup() {
				return getLibrary('library-v2');
			};

			var execute = function execute(repo) {
				return (0, _validation.validateLibrary)(repo);
			};

			var verify = function verify(result) {
				return (0, _testSetup.expect)(result.valid).to.be.true;
			};

			return setup().then(execute).then(verify);
		});

		it('returns errors for a library with invalid name', function () {
			var setup = function setup() {
				return getLibrary('library-v2-invalid');
			};

			var execute = function execute(repo) {
				return (0, _validation.validateLibrary)(repo);
			};

			var verify = function verify(result) {
				(0, _testSetup.expect)(result.valid).to.be.false;
				(0, _testSetup.expect)(result.errors).to.have.key('name');
			};
			return setup().then(execute).then(verify);
		});

		it('returns errors for a library in v1 format', function () {
			var setup = function setup() {
				return getLibrary('library-v1');
			};

			var execute = function execute(repo) {
				return (0, _validation.validateLibrary)(repo);
			};

			var verify = function verify(result) {
				(0, _testSetup.expect)(result.valid).to.be.false;
				(0, _testSetup.expect)(result.errors).to.have.key('library');
			};
			return setup().then(execute).then(verify);
		});

		it('returns errors for a directory that is not a library', function () {
			var setup = function setup() {
				return getLibrary('not-a-library');
			};

			var execute = function execute(repo) {
				return (0, _validation.validateLibrary)(repo);
			};

			var verify = function verify(result) {
				(0, _testSetup.expect)(result.valid).to.be.false;
				(0, _testSetup.expect)(result.errors).to.have.key('library');
			};
			return setup().then(execute).then(verify);
		});

		it('returns errors for a library with missing files', function () {
			var setup = function setup() {
				return getLibrary('library-v2-missing-files');
			};

			var execute = function execute(repo) {
				return (0, _validation.validateLibrary)(repo);
			};

			var verify = function verify(result) {
				(0, _testSetup.expect)(result.valid).to.be.false;
				(0, _testSetup.expect)(result.errors).to.have.keys('README.md', 'LICENSE', 'main source', 'main header');
			};
			return setup().then(execute).then(verify);
		});
	});

	describe('error handling', function () {
		var err = { msg: 'test error' };
		var repo = { getLibraryLayout: function getLibraryLayout() {
				return _promise2.default.reject(err);
			} };

		it('validateLibrary throws an error that is not ValidationFailedError', function () {
			(0, _testSetup.expect)((0, _validation.validateLibrary)(repo, 'testlib')).to.eventually.be.rejectedWith(err);
		});

		it('validateLibrary returns error results for ValidationFailedError', function () {
			var err = { msg: 'test error', name: 'LibraryNotFoundError' };
			var repo = { getLibraryLayout: function getLibraryLayout() {
					return _promise2.default.reject(err);
				} };
			var results = { errors: { library: 'is missing library.properties' }, valid: false };
			return (0, _testSetup.expect)((0, _validation.validateLibrary)(repo, 'testlib')).to.eventually.deep.equal(results);
		});
	});

	describe('error messaging', function () {
		it('formats empty errors', function () {
			var invalid = { valid: false, errors: {} };
			(0, _testSetup.expect)((0, _validation.formatValidationErrors)(invalid).join()).to.be.equal('');
		});

		it('formats a single validation error', function () {
			var invalid = { valid: false, errors: { a: 'needed b' } };
			(0, _testSetup.expect)((0, _validation.formatValidationErrors)(invalid).join()).to.be.equal('a needed b');
		});
	});
}); /*
     ******************************************************************************
     Copyright (c) 2016 Particle Industries, Inc.  All rights reserved.
    
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation, either
     version 3 of the License, or (at your option) any later version.
    
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
    
     You should have received a copy of the GNU Lesser General Public
     License along with this program; if not, see <http://www.gnu.org/licenses/>.
     ******************************************************************************
     */
//# sourceMappingURL=validation.spec.js.map